@@ -283,7 +283,7 @@ basic_engine_shape (PangoFont        *font,
 
 static PangoCoverage *
 basic_engine_get_coverage (PangoFont  *font,
-			   const char *lang)
+			   PangoLanguage *lang)
 {
   PangoCoverage *result;
 #if 0
