@@ -80,16 +80,6 @@ LevelInfo;
 static gboolean fribidi_debug = FALSE;
 #endif
 
-gboolean
-fribidi_set_debug (gboolean debug)
-{
-#ifdef DEBUG
-  return fribidi_debug = debug;
-#else
-  return 0;
-#endif
-}
-
 #ifndef USE_SIMPLE_MALLOC
 static TypeLink *free_type_links = NULL;
 #endif
@@ -596,7 +586,11 @@ fribidi_analyse_string (	/* input */
   /* Determinate character types */
   DBG ("  Determine character types\n");
   {
+#ifndef _MSC_VER /* just another gccism ... */
     FriBidiCharType char_type[len];
+#else
+    FriBidiCharType* char_type = _alloca(len*sizeof(FriBidiCharType));
+#endif
     for (i = 0; i < len; i++)
       char_type[i] = _pango_fribidi_get_type (str[i]);
 
@@ -1057,20 +1051,6 @@ free_rl_list (TypeLink *type_rl_list)
   return;
 }
 
-static gboolean mirroring = TRUE;
-
-gboolean
-fribidi_mirroring_status (void)
-{
-  return mirroring;
-}
-
-void
-fribidi_set_mirroring (gboolean mirror)
-{
-  mirroring = mirror;
-}
-
 /*======================================================================
  *  Here starts the exposed front end functions.
  *----------------------------------------------------------------------*/
