@@ -1,4 +1,4 @@
-/* Pango
+/* pango
  * pango-attributes.c: Attributed text
  *
  * Copyright (C) 2000 Red Hat Software
@@ -21,7 +21,8 @@
 
 #include <string.h>
 
-#include <pango/pango-attributes.h>
+#include "pango-attributes.h"
+#include "pango-utils.h"
 
 struct _PangoAttrList
 {
@@ -158,29 +159,6 @@ pango_attr_string_new (const PangoAttrClass *klass,
   return (PangoAttribute *)result;
 }
 
-/**
- * pango_attr_lang_new:
- * @lang: language tag (in the form "en_US")
- * 
- * Create a new language tag attribute. 
- * 
- * Return value: the new #PangoAttribute.
- **/
-PangoAttribute *
-pango_attr_lang_new (const char *lang)
-{
-  static const PangoAttrClass klass = {
-    PANGO_ATTR_LANG,
-    pango_attr_string_copy,
-    pango_attr_string_destroy,
-    pango_attr_string_equal
-  };
-
-  g_return_val_if_fail (lang != NULL, NULL);
-  
-  return pango_attr_string_new (&klass, lang);
-}
-
 /**
  * pango_attr_family_new:
  * @family: the family or comma separated list of families
@@ -204,6 +182,55 @@ pango_attr_family_new (const char *family)
   return pango_attr_string_new (&klass, family);
 }
 
+static PangoAttribute *
+pango_attr_language_copy (const PangoAttribute *attr)
+{
+  return g_memdup (attr, sizeof (PangoAttrLanguage));
+}
+
+static void
+pango_attr_language_destroy (PangoAttribute *attr)
+{
+  g_free (attr);
+}
+
+static gboolean
+pango_attr_language_equal (const PangoAttribute *attr1,
+			   const PangoAttribute *attr2)
+{
+  return ((PangoAttrLanguage *)attr1)->value == ((PangoAttrLanguage *)attr2)->value;
+}
+
+/**
+ * pango_attr_language_new:
+ * @language: language tag
+ * 
+ * Create a new language tag attribute. 
+ * 
+ * Return value: the new #PangoAttribute.
+ **/
+PangoAttribute *
+pango_attr_language_new (PangoLanguage *language)
+{
+  PangoAttrLanguage *result;
+  
+  static const PangoAttrClass klass = {
+    PANGO_ATTR_LANGUAGE,
+    pango_attr_language_copy,
+    pango_attr_language_destroy,
+    pango_attr_language_equal
+  };
+
+  g_return_val_if_fail (language != NULL, NULL);
+  
+  result = g_new (PangoAttrLanguage, 1);
+
+  result->attr.klass = &klass;
+  result->value = language;
+
+  return (PangoAttribute *)result;
+}
+
 static PangoAttribute *
 pango_attr_color_copy (const PangoAttribute *attr)
 {
@@ -691,8 +718,8 @@ pango_attr_list_get_type (void)
   if (our_type == 0)
     our_type = g_boxed_type_register_static ("PangoAttrList",
 					     NULL,
-					     pango_attr_list_copy,
-					     pango_attr_list_unref,
+					     (GBoxedCopyFunc) pango_attr_list_copy,
+					     (GBoxedFreeFunc) pango_attr_list_unref,
 					     FALSE);
 
   return our_type;
@@ -1346,18 +1373,21 @@ pango_attr_iterator_get (PangoAttrIterator *iterator,
  *           an attribute in the #PangoAttrList associated with the structure,
  *           or with @base. If you want to save this value, you should
  *           allocate it on the stack and then use pango_font_description_copy().
+ * @language: if non-%NULl, location to store language tag for item, or %NULL
+ *            if non is found.
  * @extra_attrs: if non-%NULL, location in which to store a list of non-font
  *           attributes at the the current position; only the highest priority
  *           value of each attribute will be added to this list. In order
  *           to free this value, you must call pango_attribute_destroy() on
  *           each member.
  *
- * Get the font 
+ * Get the font and other attributes at the current iterator position.
  **/
 void
 pango_attr_iterator_get_font (PangoAttrIterator     *iterator,
 			      PangoFontDescription  *base,
 			      PangoFontDescription  *current,
+			      PangoLanguage        **language,
 			      GSList               **extra_attrs)
 {
   GList *tmp_list1;
@@ -1369,6 +1399,7 @@ pango_attr_iterator_get_font (PangoAttrIterator     *iterator,
   gboolean have_weight = FALSE;
   gboolean have_stretch = FALSE;
   gboolean have_size = FALSE;
+  gboolean have_language = FALSE;
 
   g_return_if_fail (iterator != NULL);
   g_return_if_fail (base != NULL);
@@ -1376,6 +1407,9 @@ pango_attr_iterator_get_font (PangoAttrIterator     *iterator,
   
   *current = *base;
 
+  if (language)
+    *language = NULL;
+  
   if (extra_attrs)
     *extra_attrs = NULL;
   
@@ -1471,7 +1505,16 @@ pango_attr_iterator_get_font (PangoAttrIterator     *iterator,
 	      current->size = ((PangoAttrFloat *)attr)->value * base->size;
 	    }
           break;
-          
+	case PANGO_ATTR_LANGUAGE:
+	  if (language)
+	    {
+	      if (!have_language)
+		{
+		  have_language = TRUE;
+		  *language = ((PangoAttrLanguage *)attr)->value;
+		}
+	    }
+	  break;
 	default:
 	  if (extra_attrs)
 	    {
@@ -1496,57 +1539,3 @@ pango_attr_iterator_get_font (PangoAttrIterator     *iterator,
 	}
     }
 }
-
-GType
-pango_color_get_type (void)
-{
-  static GType our_type = 0;
-  
-  if (our_type == 0)
-    our_type = g_boxed_type_register_static ("PangoColor",
-					     NULL,
-					     pango_color_copy,
-					     pango_color_free,
-					     FALSE);
-
-  return our_type;
-}
-
-/**
- * pango_color_copy:
- * @src: color to copy
- * 
- * Creates a copy of @src, which should be freed with
- * pango_color_free(). Primarily used by language bindings,
- * not that useful otherwise (since colors can just be copied
- * by assignment in C).
- * 
- * Return value: an allocated #PangoColor
- **/
-PangoColor*
-pango_color_copy (const PangoColor *src)
-{
-  PangoColor *ret;
-
-  g_return_val_if_fail (src != NULL, NULL);
-  
-  ret = g_new (PangoColor, 1);
-
-  *ret = *src;
-
-  return ret;
-}
-
-/**
- * pango_color_free:
- * @color: an allocated #PangoColor
- * 
- * Frees a color allocated by pango_color_copy().
- **/
-void
-pango_color_free (PangoColor *color)
-{
-  g_return_if_fail (color != NULL);
-
-  g_free (color);
-}
