@@ -31,7 +31,7 @@ struct _PangoContext
 {
   GObject parent_instance;
 
-  char *lang;
+  PangoLanguage *language;
   PangoDirection base_dir;
   PangoFontDescription *font_desc;
 
@@ -51,10 +51,7 @@ static void add_engines (PangoContext      *context,
 			 PangoAttrList     *attrs,
                          PangoAttrIterator *cached_iter,
                          gint               n_chars,
-			 PangoEngineShape **shape_engines,
-			 PangoEngineLang  **lang_engines,
-			 PangoFont        **fonts,
-			 GSList           **extra_attr_lists);
+			 PangoAnalysis     *analyses);
 
 static void pango_context_init        (PangoContext      *context);
 static void pango_context_class_init  (PangoContextClass *klass);
@@ -96,7 +93,7 @@ pango_context_init (PangoContext *context)
   PangoFontDescription desc;  
 
   context->base_dir = PANGO_DIRECTION_LTR;
-  context->lang = NULL;
+  context->language = NULL;
   context->font_maps = NULL;
 
   desc.family_name = "serif";
@@ -126,9 +123,6 @@ pango_context_finalize (GObject *object)
 
   context = PANGO_CONTEXT (object);
 
-  if (context->lang)
-    g_free (context->lang);
-  
   g_slist_foreach (context->font_maps, (GFunc)g_object_unref, NULL);
   g_slist_free (context->font_maps);
 
@@ -150,8 +144,6 @@ pango_context_new (void)
 {
   PangoContext *context;
 
-  g_type_init ();
-  
   context = PANGO_CONTEXT (g_type_create_instance (pango_context_get_type ()));
   
   return context;
@@ -431,38 +423,35 @@ pango_context_get_font_description (PangoContext *context)
 }
 
 /**
- * pango_context_set_lang:
+ * pango_context_set_language:
  * @context: a #PangoContext
- * @lang: the new language tag.
+ * @language: the new language tag.
  * 
  * Sets the global language tag for the context.
  **/
 void
-pango_context_set_lang (PangoContext *context,
-			const char   *lang)
+pango_context_set_language (PangoContext *context,
+			    PangoLanguage    *language)
 {
   g_return_if_fail (context != NULL);
 
-  if (context->lang)
-    g_free (context->lang);
-
-  context->lang = g_strdup (lang);
+  context->language = language;
 }
 
 /**
- * pango_context_get_lang:
+ * pango_context_get_language:
  * @context: a #PangoContext
  * 
  * Retrieves the global language tag for the context.
  * 
- * Return value: the global language tag. This value must be freed with g_free().
+ * Return value: the global language tag.
  **/
-char *
-pango_context_get_lang (PangoContext *context)
+PangoLanguage *
+pango_context_get_language (PangoContext *context)
 {
   g_return_val_if_fail (context != NULL, NULL);
 
-  return g_strdup (context->lang);
+  return context->language;
 }
 
 /**
@@ -510,7 +499,10 @@ pango_context_get_base_dir (PangoContext *context)
  * @cached_iter:      Cached attribute iterator, or NULL
  *
  * Breaks a piece of text into segments with consistent
- * directional level and shaping engine.
+ * directional level and shaping engine. Each byte of @text will
+ * be contained in exactly one of the items in the returned list;
+ * the generated list of items will be in logical order (the start
+ * offsets of the items are ascending).
  *
  * @cached_iter should be an iterator over @attrs currently positioned at a
  * range before or containing @start_index; @cached_iter will be advanced to
@@ -528,18 +520,15 @@ pango_itemize (PangoContext      *context,
                PangoAttrIterator *cached_iter)
 {
   gunichar *text_ucs4;
-  int n_chars, i;
+  long n_chars, i;
   guint8 *embedding_levels;
   PangoDirection base_dir;
   PangoItem *item;
   const char *p;
   const char *next;
   GList *result = NULL;
-  
-  PangoEngineShape **shape_engines;
-  PangoEngineLang  **lang_engines;
-  GSList           **extra_attr_lists;
-  PangoFont        **fonts;
+
+  PangoAnalysis *analyses;
 
   g_return_val_if_fail (context != NULL, NULL);
   g_return_val_if_fail (start_index >= 0, NULL);
@@ -557,24 +546,18 @@ pango_itemize (PangoContext      *context,
   /* First, apply the bidirectional algorithm to break
    * the text into directional runs.
    */
-  text_ucs4 = g_utf8_to_ucs4 (text + start_index, length);
-  if (!text_ucs4)
-    return NULL;
+  text_ucs4 = g_utf8_to_ucs4_fast (text + start_index, length, &n_chars);
 
-  n_chars = g_utf8_strlen (text + start_index, length);
   embedding_levels = g_new (guint8, n_chars);
 
   pango_log2vis_get_embedding_levels (text_ucs4, n_chars, &base_dir,
-					embedding_levels);
+                                      embedding_levels);
 
-  /* Storing these as ranges would be a lot more efficient,
-   * but also more complicated... we take the simple
-   * approach for now.
+  /* Storing ranges would be more efficient, but also more
+   * complicated... we take the simple approach for now.
    */
-  shape_engines = g_new0 (PangoEngineShape *, n_chars);
-  lang_engines = g_new0 (PangoEngineLang *, n_chars);
-  fonts = g_new0 (PangoFont *, n_chars);
-  extra_attr_lists = g_new0 (GSList *, n_chars);
+
+  analyses = g_new0 (PangoAnalysis, n_chars);
 
   /* Now, fill in the appropriate shapers, language engines and fonts for
    * each character.
@@ -583,8 +566,7 @@ pango_itemize (PangoContext      *context,
   add_engines (context, text, start_index, length, attrs,
                cached_iter,
                n_chars,
-               shape_engines, lang_engines, fonts,
-	       extra_attr_lists);
+	       analyses);
 
   /* Make a GList of PangoItems out of the above results
    */
@@ -593,30 +575,39 @@ pango_itemize (PangoContext      *context,
   p = text + start_index;
   for (i=0; i<n_chars; i++)
     {
+      PangoAnalysis *analysis = &analyses[i];
+      PangoAnalysis *last_analysis = i > 0 ? &analyses[i-1] : 0;
+      
       next = g_utf8_next_char (p);
       
       if (i == 0 ||
 	  text_ucs4[i] == '\t' || text_ucs4[i-1] == '\t' ||
 	  embedding_levels[i] != embedding_levels[i-1] ||
-	  shape_engines[i] != shape_engines[i-1] ||
-	  lang_engines[i] != lang_engines[i-1] ||
-	  fonts[i] != fonts[i-1] ||
-	  extra_attr_lists[i] != extra_attr_lists[i-1])
+	  analysis->shape_engine != last_analysis->shape_engine ||
+	  analysis->lang_engine != last_analysis->lang_engine ||
+	  analysis->font != last_analysis->font ||
+	  analysis->language != last_analysis->language ||
+	  analysis->extra_attrs != last_analysis->extra_attrs)
 	{
-	  item = g_new (PangoItem, 1);
+          /* assert that previous item got at least one char */
+          g_assert (item == NULL || item->length > 0);
+          g_assert (item == NULL || item->num_chars > 0);
+          
+	  item = pango_item_new ();
 	  item->offset = p - text;
 	  item->num_chars = 0;
 	  item->analysis.level = embedding_levels[i];
 	  
-	  item->analysis.shape_engine = shape_engines[i];
-	  item->analysis.lang_engine = lang_engines[i];
+	  item->analysis.shape_engine = analysis->shape_engine;
+	  item->analysis.lang_engine = analysis->lang_engine;
 
-	  item->analysis.font = fonts[i];
+	  item->analysis.font = analysis->font;
+	  item->analysis.language = analysis->language;
 
 	  /* Copy the extra attribute list if necessary */
-	  if (extra_attr_lists[i] && i != 0 && extra_attr_lists[i] == extra_attr_lists[i-1])
+	  if (analysis->extra_attrs && i != 0 && analysis->extra_attrs == last_analysis->extra_attrs)
 	    {
-	      GSList *tmp_list = extra_attr_lists[i];
+	      GSList *tmp_list = analysis->extra_attrs;
 	      GSList *new_list = NULL;
 	      while (tmp_list)
 		{
@@ -624,160 +615,228 @@ pango_itemize (PangoContext      *context,
 					      pango_attribute_copy (tmp_list->data));
 		  tmp_list = tmp_list->next;
 		}
-	      item->extra_attrs = g_slist_reverse (new_list);
+	      item->analysis.extra_attrs = g_slist_reverse (new_list);
 	    }
 	  else
-	    item->extra_attrs = extra_attr_lists[i];
+	    item->analysis.extra_attrs = analysis->extra_attrs;
 
 	  result = g_list_prepend (result, item);
 	}
       else
-	g_object_unref (G_OBJECT (fonts[i]));
+	g_object_unref (analysis->font);
 
       item->length = (next - text) - item->offset;
       item->num_chars++;
       p = next;
     }  
 
+  g_free (analyses);
   g_free (embedding_levels);
-  g_free (shape_engines);
-  g_free (lang_engines);
-  g_free (fonts);
-  g_free (extra_attr_lists);
-  
   g_free (text_ucs4);
   
   return g_list_reverse (result);
 }
 
-static PangoFont *
-get_font (PangoFont     **fonts,
-	  PangoCoverage **coverages,
-	  int             n_families,
-	  gunichar        wc)
+static void 
+fallback_engine_shape (PangoFont        *font,
+                       const char       *text,
+                       gint              length,
+                       PangoAnalysis    *analysis,
+                       PangoGlyphString *glyphs)
+{
+  int n_chars;
+  int i;
+  const char *p;
+  
+  g_return_if_fail (font != NULL);
+  g_return_if_fail (text != NULL);
+  g_return_if_fail (length >= 0);
+  g_return_if_fail (analysis != NULL);
+
+  n_chars = g_utf8_strlen (text, length);
+  pango_glyph_string_set_size (glyphs, n_chars);
+  
+  p = text;
+  i = 0;
+  while (i < n_chars)
+    {
+      glyphs->glyphs[i].glyph = 0;
+      
+      glyphs->glyphs[i].geometry.x_offset = 0;
+      glyphs->glyphs[i].geometry.y_offset = 0;
+      glyphs->glyphs[i].geometry.width = 0;
+
+      glyphs->log_clusters[i] = p - text;
+      
+      ++i;
+      p = g_utf8_next_char (p);
+    }
+}
+
+static PangoCoverage*
+fallback_engine_get_coverage (PangoFont      *font,
+                              PangoLanguage  *lang)
+{
+  PangoCoverage *result = pango_coverage_new ();
+
+  /* We return an empty coverage (this function won't get
+   * called, but if it is, empty coverage will keep
+   * it from being used).
+   */
+  
+  return result;
+}
+
+static PangoEngineShape fallback_shaper = {
+  {
+    "FallbackScriptEngine",
+    PANGO_ENGINE_TYPE_SHAPE,
+    sizeof (PangoEngineShape)
+  },
+  fallback_engine_shape,
+  fallback_engine_get_coverage
+};
+
+/* FIXME: Remove this artificial limit */
+#define MAX_FAMILIES 16
+
+typedef struct _FontSet FontSet;
+
+struct _FontSet
+{
+  int n_families;
+  PangoFont *fonts[MAX_FAMILIES];
+  PangoCoverage *coverages[MAX_FAMILIES];  
+};
+
+#define FONT_SET_INITIALIZER { 0, }
+
+static gint
+font_set_get_font (FontSet  *font_set,
+		   gunichar  wc)
 {
-  PangoFont *result = NULL;
   PangoCoverageLevel best_level = PANGO_COVERAGE_NONE;
 
+  int result = -1;
   int i;
   
-  for (i=0; i<n_families; i++)
+  for (i=0; i < font_set->n_families; i++)
     {
-      if (fonts[i])
+      if (font_set->fonts[i])
 	{
-	  PangoCoverageLevel level = pango_coverage_get (coverages[i], wc);
+	  PangoCoverageLevel level = pango_coverage_get (font_set->coverages[i], wc);
 	  
-	  if (!result || level > best_level)
+	  if (result == -1 || level > best_level)
 	    {
-	      result = fonts[i];
+	      result = i;
 	      best_level = level;
 	    }
 	}
     }
 
-  if (result)
-    g_object_ref (G_OBJECT (result));
-      
   return result;
 }
 
-/* FIXME: Remove this artificial limit */
-#define MAX_FAMILIES 16
-
 static void
-load_font (PangoContext          *context,
-	   char                  *lang,
-	   PangoFontDescription  *desc,
-	   PangoFont            **current_fonts,
-	   PangoCoverage        **current_coverages,
-	   int                   *n_families)
+font_set_free (FontSet *font_set)
 {
-  char **families;
-  char *orig_family;
   int j;
   
-  for (j=0; j < *n_families; j++)
+  for (j=0; j < font_set->n_families; j++)
     {
-      if (current_fonts[j])
+      if (font_set->fonts[j])
 	{
-	  g_object_unref (G_OBJECT (current_fonts[j]));
-	  pango_coverage_unref (current_coverages[j]);
+	  g_object_unref (font_set->fonts[j]);
+	  pango_coverage_unref (font_set->coverages[j]);
 	}
     }
 
-  orig_family = desc->family_name;
-  families = g_strsplit (orig_family, ",", -1);
+  font_set->n_families = 0;
+}
 
-  *n_families = 0;
-  for (j=0; families[j] && *n_families < MAX_FAMILIES; j++)
+static void
+font_set_load (FontSet              *font_set,
+	       PangoContext         *context,
+	       PangoLanguage        *language,
+	       PangoFontDescription *desc)
+{
+  PangoFontDescription tmp_desc = *desc;
+  char **families;
+  int j;
+
+  font_set_free (font_set);
+
+  families = g_strsplit (desc->family_name, ",", -1);
+
+  font_set->n_families = 0;
+  for (j=0; families[j] && font_set->n_families < MAX_FAMILIES; j++)
     {
-      desc->family_name = families[j];
-      current_fonts[*n_families] = pango_context_load_font (context, desc);
+      tmp_desc.family_name = families[j];
+      font_set->fonts[font_set->n_families] = pango_context_load_font (context, &tmp_desc);
       
-      if (current_fonts[*n_families])
+      if (font_set->fonts[font_set->n_families])
 	{
-	  current_coverages[*n_families] = pango_font_get_coverage (current_fonts[*n_families], lang);
-	  (*n_families)++;
+	  font_set->coverages[font_set->n_families] = pango_font_get_coverage (font_set->fonts[font_set->n_families], language);
+	  (font_set->n_families)++;
 	}
     }
   
   g_strfreev (families);
+  tmp_desc.family_name = desc->family_name;
 
   /* The font description was completely unloadable, try with
    * family == "Sans"
    */
-  if (*n_families == 0)
+  if (font_set->n_families == 0)
     {
       char *ctmp1, *ctmp2;
       
-      desc->family_name = orig_family;
-      
       ctmp1 = pango_font_description_to_string (desc);
-      desc->family_name = "Sans";
-      ctmp2 = pango_font_description_to_string (desc);
+      tmp_desc.family_name = "Sans";
+      ctmp2 = pango_font_description_to_string (&tmp_desc);
       
       g_warning ("Couldn't load font \"%s\" falling back to \"%s\"", ctmp1, ctmp2);
       g_free (ctmp1);
       g_free (ctmp2);
       
-      desc->family_name = "Sans";
+      tmp_desc.family_name = "Sans";
       
-      current_fonts[0] = pango_context_load_font (context, desc);
-      if (current_fonts[0])
+      font_set->fonts[0] = pango_context_load_font (context, &tmp_desc);
+      if (font_set->fonts[0])
 	{
-	  current_coverages[0] = pango_font_get_coverage (current_fonts[0], lang);
-	  *n_families = 1;
+	  font_set->coverages[0] = pango_font_get_coverage (font_set->fonts[0], language);
+	  font_set->n_families = 1;
 	}
     }
   
   /* We couldn't try with Sans and the specified style. Try Sans Normal
    */
-  if (*n_families == 0)
+  if (font_set->n_families == 0)
     {
       char *ctmp1, *ctmp2;
       
-      ctmp1 = pango_font_description_to_string (desc);
-      desc->style = PANGO_STYLE_NORMAL;
-      desc->weight = PANGO_WEIGHT_NORMAL;
-      desc->variant = PANGO_VARIANT_NORMAL;
-      desc->stretch = PANGO_STRETCH_NORMAL;
-      ctmp2 = pango_font_description_to_string (desc);
+      ctmp1 = pango_font_description_to_string (&tmp_desc);
+      tmp_desc.style = PANGO_STYLE_NORMAL;
+      tmp_desc.weight = PANGO_WEIGHT_NORMAL;
+      tmp_desc.variant = PANGO_VARIANT_NORMAL;
+      tmp_desc.stretch = PANGO_STRETCH_NORMAL;
+      ctmp2 = pango_font_description_to_string (&tmp_desc);
       
       g_warning ("Couldn't load font \"%s\" falling back to \"%s\"", ctmp1, ctmp2);
       g_free (ctmp1);
       g_free (ctmp2);
       
-      current_fonts[0] = pango_context_load_font (context, desc);
-      if (current_fonts[0])
+      font_set->fonts[0] = pango_context_load_font (context, &tmp_desc);
+      if (font_set->fonts[0])
 	{
-	  current_coverages[0] = pango_font_get_coverage (current_fonts[0], lang);
-	  *n_families = 1;
+	  font_set->coverages[0] = pango_font_get_coverage (font_set->fonts[0], language);
+	  font_set->n_families = 1;
 	}
     }
 
   /* Everything failed, we are screwed, there is no way to continue
    */
-  if (n_families == 0)
+  if (font_set->n_families == 0)
     {
       g_warning ("All font failbacks failed!!!!");
       exit (1);
@@ -814,25 +873,20 @@ add_engines (PangoContext      *context,
 	     PangoAttrList     *attrs,
              PangoAttrIterator *cached_iter,
              gint               n_chars,
-	     PangoEngineShape **shape_engines,
-	     PangoEngineLang  **lang_engines,
-	     PangoFont        **fonts,
-	     GSList           **extra_attr_lists)
+	     PangoAnalysis     *analyses)
 {
   const char *pos;
-  char *lang = NULL;
+  PangoLanguage *language = NULL;
   int next_index;
   GSList *extra_attrs = NULL;
   PangoMap *lang_map = NULL;
   PangoFontDescription current_desc = { 0 };
-  int n_families = 0;
-  PangoFont *current_fonts[MAX_FAMILIES];
-  PangoCoverage *current_coverages[MAX_FAMILIES];  
+  FontSet current_fonts = FONT_SET_INITIALIZER;
   PangoAttrIterator *iterator;
-  PangoAttribute *attr;
   gboolean first_iteration = TRUE;
   gunichar wc;
-  int i = 0, j;
+  int i = 0;
+  int font_index;
 
   if (cached_iter)
     iterator = cached_iter;
@@ -846,9 +900,11 @@ add_engines (PangoContext      *context,
   pos = text + start_index;
   for (i=0; i<n_chars; i++)
     {
+      PangoAnalysis *analysis = &analyses[i];
+      
       if (first_iteration || pos - text == next_index)
 	{
-	  char *next_lang;
+	  PangoLanguage *next_language;
 	  PangoFontDescription next_desc;
 
           first_iteration = FALSE;
@@ -862,20 +918,18 @@ add_engines (PangoContext      *context,
               pango_attr_iterator_range (iterator, NULL, &next_index);
             }
           
-	  attr = pango_attr_iterator_get (iterator, PANGO_ATTR_LANG);
-	  if (attr)
-	    next_lang = ((PangoAttrString *)attr)->value;
-	  else
-	    next_lang = context->lang;
+	  pango_attr_iterator_get_font (iterator, context->font_desc,
+                                        &next_desc, &next_language, &extra_attrs);
 
-	  if (i == 0 ||
-	      (lang != next_lang &&
-	       (lang == NULL || next_lang == NULL || strcmp (lang, next_lang) != 0)))
+          if (!next_language)
+	    next_language = context->language;
+
+	  if (i == 0 || language != next_language)
 	    {
 	      static guint engine_type_id = 0;
 	      static guint render_type_id = 0;
 	      
-	      lang = next_lang;
+	      language = next_language;
 
 	      if (engine_type_id == 0)
 		{
@@ -883,50 +937,134 @@ add_engines (PangoContext      *context,
 		  render_type_id = g_quark_from_static_string (PANGO_RENDER_TYPE_NONE);
 		}
 	       
-	      lang_map = pango_find_map (next_lang,
+	      lang_map = pango_find_map (next_language,
 					 engine_type_id, render_type_id);
 	    }
 
-	  pango_attr_iterator_get_font (iterator, context->font_desc,
-                                        &next_desc, &extra_attrs);
-          
 	  if (i == 0 ||
 	      !pango_font_description_equal (&current_desc, &next_desc))
 	    {
 	      current_desc = next_desc;
 
-	      load_font (context, lang, &current_desc,
-			 current_fonts, current_coverages, &n_families);
+	      font_set_load (&current_fonts, context, language, &current_desc);
 	    }
         }
 
       wc = g_utf8_get_char (pos);
       pos = g_utf8_next_char (pos);
-	  
-      lang_engines[i] = (PangoEngineLang *)pango_map_get_engine (lang_map, wc);
-      fonts[i] = get_font (current_fonts, current_coverages, n_families, wc);
-
-      /* FIXME: handle reference counting properly on the shapers */
-      if (fonts[i])
-	shape_engines[i] = pango_font_find_shaper (fonts[i], lang, wc);
+      
+      analysis->lang_engine = (PangoEngineLang *)pango_map_get_engine (lang_map, wc);
+      font_index = font_set_get_font (&current_fonts, wc);
+      if (font_index != -1)
+	{
+	  analysis->font = current_fonts.fonts[font_index];
+	  g_object_ref (analysis->font);
+	}
       else
-	shape_engines[i] = NULL;
-
-      extra_attr_lists[i] = extra_attrs;
+	analysis->font = NULL;
+      analysis->language = language;
+      
+      /* FIXME: handle reference counting properly on the shapers */
+      if (analysis->font)
+	analysis->shape_engine = pango_font_find_shaper (analysis->font, language, wc);
+      else
+	analysis->shape_engine = NULL;
+      
+      if (analysis->shape_engine == NULL)
+        analysis->shape_engine = &fallback_shaper;
+      
+      analysis->extra_attrs = extra_attrs;
     }
 
   g_assert (pos - text == start_index + length);
-  
-  for (j=0; j<n_families; j++)
-    {
-      if (current_fonts[j])
-	{
-	  g_object_unref (G_OBJECT (current_fonts[j]));
-	  pango_coverage_unref (current_coverages[j]);
-	}
-    }
+
+  font_set_free (&current_fonts);
 
   if (iterator != cached_iter)
     pango_attr_iterator_destroy (iterator);
 }
 
+/**
+ * pango_context_get_metrics:
+ * @context: a #PangoContext
+ * @desc: a #PangoFontDescription structure
+ * @language: language tag used to determine which script to get the metrics
+ *            for, or %NULL to indicate to get the metrics for the entire
+ *            font.
+ * @metrics: Structure to fill in with the metrics of the font
+ * 
+ * Get overall metric information for a font particular font
+ * description.  Since the metrics may be substantially different for
+ * different scripts, a language tag can be provided to indicate that
+ * the metrics should be retrieved that correspond to the script(s)
+ * used by that language.
+ *
+ * The #PangoFontDescription is interpreted in the same way as
+ * by pango_itemize(), and the family name may be a comma separated
+ * list of figures. If characters from multiple of these families
+ * would be used to render the string, then the returned fonts would
+ * be a composite of the metrics for the fonts loaded for the
+ * individual families.
+ **/
+void
+pango_context_get_metrics (PangoContext                 *context,
+			   const PangoFontDescription   *desc,
+			   PangoLanguage                *language,
+			   PangoFontMetrics             *metrics)
+{
+  FontSet current_fonts = FONT_SET_INITIALIZER;
+  PangoFontMetrics raw_metrics[MAX_FAMILIES];
+  gboolean have_metrics[MAX_FAMILIES];
+  PangoFontDescription tmp_desc = *desc;
+  const char *sample_str;
+  const char *p;
+  int i;
+
+  g_return_if_fail (PANGO_IS_CONTEXT (context));
+  g_return_if_fail (desc != NULL);
+  g_return_if_fail (metrics != NULL);
+
+  sample_str = pango_language_get_sample_string (language);
+
+  font_set_load (&current_fonts, context, language, &tmp_desc);
+
+  for (i=0; i < MAX_FAMILIES; i++)
+    have_metrics[i] = FALSE;
+  
+  if (current_fonts.n_families == 1)
+    pango_font_get_metrics (current_fonts.fonts[0], language, metrics);
+  else
+    {
+      int count = 0;
+
+      p = sample_str;
+      while (*p)
+	{
+	  gunichar wc = g_utf8_get_char (p);
+	  int index = font_set_get_font (&current_fonts, wc);
+	  if (!have_metrics[index])
+	    {
+	      pango_font_get_metrics (current_fonts.fonts[index], language, &raw_metrics[index]);
+	      have_metrics[index] = TRUE;
+	    }
+
+	  if (count == 0)
+	    *metrics = raw_metrics[index];
+	  else
+	    {
+	      metrics->ascent = MAX (metrics->ascent, raw_metrics[index].ascent);
+	      metrics->descent = MAX (metrics->descent, raw_metrics[index].descent);
+	      metrics->approximate_char_width += raw_metrics[index].approximate_char_width;
+	      metrics->approximate_digit_width += raw_metrics[index].approximate_digit_width;
+	    }
+	  
+	  p = g_utf8_next_char (p);
+	  count++;
+	}
+
+      metrics->approximate_char_width /= count;
+      metrics->approximate_digit_width /= count;
+    }
+      
+  font_set_free (&current_fonts);
+}
