@@ -23,6 +23,7 @@
 #define __PANGO_TYPES_H__
 
 #include <glib.h>
+#include <glib-object.h>
 
 typedef struct _PangoLangRange PangoLangRange;
 typedef struct _PangoLogAttr PangoLogAttr;
@@ -33,6 +34,9 @@ typedef struct _PangoEngineShape PangoEngineShape;
 typedef struct _PangoFont PangoFont;
 typedef struct _PangoRectangle PangoRectangle;
 
+/* Dummy typedef - internally it's a 'const char *' */
+typedef struct _PangoLanguage PangoLanguage;
+
 /* A index of a glyph into a font. Rendering system dependent
  */
 typedef guint32 PangoGlyph;
@@ -71,14 +75,14 @@ typedef enum {
   PANGO_DIRECTION_TTB_RTL
 } PangoDirection;
 
-/* Language tagging information
- */
-struct _PangoLangRange
-{
-  gint start;
-  gint length;
-  gchar *lang;
-};
+#define PANGO_TYPE_LANGUAGE (pango_language_get_type ())
+
+GType          pango_language_get_type    (void);
+PangoLanguage *pango_language_from_string (const char *language);
+#define pango_language_to_string(language) ((const char *)language)
+
+gboolean      pango_language_matches  (PangoLanguage *language,
+				       const char *range_list);
 
 #ifdef __cplusplus
 extern "C" {
