@@ -116,7 +116,7 @@ struct _PangoXSubfontInfo
 
 struct _PangoXMetricsInfo
 {
-  const char *lang;
+  const char *sample_str;
   PangoFontMetrics metrics;
 };
 
@@ -153,21 +153,21 @@ static PangoFontClass *parent_class;	/* Parent class structure for PangoXFont */
 
 static void pango_x_font_class_init (PangoXFontClass *class);
 static void pango_x_font_init       (PangoXFont      *xfont);
-static void pango_x_font_shutdown   (GObject         *object);
+static void pango_x_font_dispose    (GObject         *object);
 static void pango_x_font_finalize   (GObject         *object);
 
 static PangoFontDescription *pango_x_font_describe          (PangoFont        *font);
 static PangoCoverage *       pango_x_font_get_coverage      (PangoFont        *font,
-							     const char       *lang);
+							     PangoLanguage    *language);
 static PangoEngineShape *    pango_x_font_find_shaper       (PangoFont        *font,
-							     const char       *lang,
+							     PangoLanguage    *language,
 							     guint32           ch);
 static void                  pango_x_font_get_glyph_extents (PangoFont        *font,
 							     PangoGlyph        glyph,
 							     PangoRectangle   *ink_rect,
 							     PangoRectangle   *logical_rect);
 static void                  pango_x_font_get_metrics       (PangoFont        *font,
-							     const gchar      *lang,
+							     PangoLanguage    *language,
 							     PangoFontMetrics *metrics);
 
 static PangoXSubfontInfo * pango_x_find_subfont    (PangoFont          *font,
@@ -358,7 +358,7 @@ pango_x_font_class_init (PangoXFontClass *class)
   parent_class = g_type_class_peek_parent (class);
   
   object_class->finalize = pango_x_font_finalize;
-  object_class->shutdown = pango_x_font_shutdown;
+  object_class->dispose = pango_x_font_dispose;
   
   font_class->describe = pango_x_font_describe;
   font_class->get_coverage = pango_x_font_get_coverage;
@@ -770,7 +770,7 @@ get_font_metrics_from_subfonts (PangoFont        *font,
  */
 static void
 get_font_metrics_from_string (PangoFont        *font,
-			      const char       *lang,
+			      PangoLanguage    *language,
 			      const char       *str,
 			      PangoFontMetrics *metrics)
 {
@@ -779,7 +779,7 @@ get_font_metrics_from_string (PangoFont        *font,
   PangoEngineShape *shaper, *last_shaper;
   int last_level;
   gunichar *text_ucs4;
-  int n_chars, i;
+  long n_chars, i;
   guint8 *embedding_levels;
   PangoDirection base_dir = PANGO_DIRECTION_LTR;
   GSList *subfonts = NULL;
@@ -806,7 +806,7 @@ get_font_metrics_from_string (PangoFont        *font,
       if (*p)
 	{
 	  wc = g_utf8_get_char (p);
-	  shaper = pango_font_find_shaper (font, lang, wc);
+	  shaper = pango_font_find_shaper (font, language, wc);
 	}
       else
 	{
@@ -824,7 +824,9 @@ get_font_metrics_from_string (PangoFont        *font,
 	  analysis.shape_engine = last_shaper;
 	  analysis.lang_engine = NULL;
 	  analysis.font = font;
+	  analysis.language = language;
 	  analysis.level = last_level;
+	  analysis.extra_attrs = NULL;
 	  
 	  pango_shape (start, p - start, &analysis, glyph_str);
 
@@ -853,95 +855,23 @@ get_font_metrics_from_string (PangoFont        *font,
   g_free (embedding_levels);
 }
 
-typedef struct {
-  const char *lang;
-  const char *str;
-} LangInfo;
-
-int
-lang_info_compare (const void *key, const void *val)
-{
-  const LangInfo *lang_info = val;
-
-  return strncmp (key, lang_info->lang, 2);
-}
-
-/* The following array is supposed to contain enough text to tickle all necessary fonts for each
- * of the languages in the following. Yes, it's pretty lame. Not all of the languages
- * in the following have sufficient text to excercise all the accents for the language, and
- * there are obviously many more languages to include as well.
- */
-LangInfo lang_texts[] = {
-  { "ar", "Arabic  السلام عليكم" },
-  { "cs", "Czech (česky)  Dobrý den" },
-  { "da", "Danish (Dansk)  Hej, Goddag" },
-  { "el", "Greek (Ελληνικά) Γειά σας" },
-  { "en", "English Hello" },
-  { "eo", "Esperanto Saluton" },
-  { "es", "Spanish (Español) ¡Hola!" },
-  { "et", "Estonian  Tere, Tervist" },
-  { "fi", "Finnish (Suomi)  Hei, Hyvää päivää" },
-  { "fr", "French (Français)" },
-  { "de", "German Grüß Gott" },
-  { "iw", "Hebrew   שלום" },
-  { "il", "Italiano  Ciao, Buon giorno" },
-  { "ja", "Japanese (日本語) こんにちは, ｺﾝﾆﾁﾊ" },
-  { "ko", "Korean (한글)   안녕하세요, 안녕하십니까" },
-  { "mt", "Maltese   Ċaw, Saħħa" },
-  { "nl", "Nederlands, Vlaams Hallo, Dag" },
-  { "no", "Norwegian (Norsk) Hei, God dag" },
-  { "pl", "Polish   Dzień dobry, Hej" },
-  { "ru", "Russian (Русский)" },
-  { "sk", "Slovak   Dobrý deň" },
-  { "sv", "Swedish (Svenska) Hej på dej, Goddag" },
-  { "tr", "Turkish (Türkçe) Merhaba" },
-  { "zh", "Chinese (中文,普通话,汉语)" }
-};
-
 static void
 pango_x_font_get_metrics (PangoFont        *font,
-			  const gchar      *lang,
+			  PangoLanguage    *language,
 			  PangoFontMetrics *metrics)
 {
   PangoXMetricsInfo *info = NULL; /* Quiet gcc */
   PangoXFont *xfont = (PangoXFont *)font;
   GSList *tmp_list;
       
-  const char *lookup_lang;
-  const char *str;
-
-  if (lang)
-    {
-      LangInfo *lang_info = bsearch (lang, lang_texts,
-				     G_N_ELEMENTS (lang_texts), sizeof (LangInfo),
-				     lang_info_compare);
-
-      if (lang_info)
-	{
-	  lookup_lang = lang_info->lang;
-	  str = lang_info->str;
-	}
-      else
-	{
-	  lookup_lang = "UNKNOWN";
-	  str = "French (Français)";     /* Assume iso-8859-1 */
-	}
-    }
-  else
-    {
-      lookup_lang = "NONE";
-
-      /* Complete junk
-       */
-      str = "السلام عليكم česky Ελληνικά Français 日本語 한글 Русский 中文,普通话,汉语 Türkçe";
-    }
+  const char *sample_str = pango_language_get_sample_string (language);
   
   tmp_list = xfont->metrics_by_lang;
   while (tmp_list)
     {
       info = tmp_list->data;
       
-      if (info->lang == lookup_lang)        /* We _don't_ need strcmp */
+      if (info->sample_str == sample_str)    /* We _don't_ need strcmp */
 	break;
 
       tmp_list = tmp_list->next;
@@ -954,18 +884,18 @@ pango_x_font_get_metrics (PangoFont        *font,
       PangoContext *context;
       
       info = g_new (PangoXMetricsInfo, 1);
-      info->lang = lookup_lang;
+      info->sample_str = sample_str;
 
       xfont->metrics_by_lang = g_slist_prepend (xfont->metrics_by_lang, info);
       
-      get_font_metrics_from_string (font, lookup_lang, str, &info->metrics);
+      get_font_metrics_from_string (font, language, sample_str, &info->metrics);
 
       /* This is sort of a sledgehammer solution, but we cache this
        * stuff so not a huge deal hopefully. Get the avg. width of the
        * chars in "0123456789"
        */
       context = pango_x_get_context (pango_x_fontmap_get_display (xfont->fontmap));
-      pango_context_set_lang (context, lookup_lang);
+      pango_context_set_language (context, language);
       layout = pango_layout_new (context);
       pango_layout_set_text (layout, "0123456789", -1);
 
@@ -1237,7 +1167,7 @@ pango_x_font_subfont_xlfd (PangoFont     *font,
 }
 
 static void
-pango_x_font_shutdown (GObject *object)
+pango_x_font_dispose (GObject *object)
 {
   PangoXFont *xfont = PANGO_X_FONT (object);
 
@@ -1248,7 +1178,7 @@ pango_x_font_shutdown (GObject *object)
   if (!xfont->in_cache && xfont->fontmap)
     pango_x_fontmap_cache_add (xfont->fontmap, xfont);
 
-  G_OBJECT_CLASS (parent_class)->shutdown (object);
+  G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
 
@@ -1326,7 +1256,7 @@ pango_x_font_describe (PangoFont *font)
 }
 
 PangoMap *
-pango_x_get_shaper_map (const char *lang)
+pango_x_get_shaper_map (PangoLanguage *language)
 {
   static guint engine_type_id = 0;
   static guint render_type_id = 0;
@@ -1337,26 +1267,26 @@ pango_x_get_shaper_map (const char *lang)
       render_type_id = g_quark_from_static_string (PANGO_RENDER_TYPE_X);
     }
   
-  return pango_find_map (lang, engine_type_id, render_type_id);
+  return pango_find_map (language, engine_type_id, render_type_id);
 }
 
 static PangoCoverage *
-pango_x_font_get_coverage (PangoFont  *font,
-			   const char *lang)
+pango_x_font_get_coverage (PangoFont     *font,
+			   PangoLanguage *language)
 {
   PangoXFont *xfont = (PangoXFont *)font;
 
-  return pango_x_font_entry_get_coverage (xfont->entry, font, lang);
+  return pango_x_font_entry_get_coverage (xfont->entry, font, language);
 }
 
 static PangoEngineShape *
-pango_x_font_find_shaper (PangoFont   *font,
-			  const gchar *lang,
-			  guint32      ch)
+pango_x_font_find_shaper (PangoFont     *font,
+			  PangoLanguage *language,
+			  guint32        ch)
 {
   PangoMap *shape_map = NULL;
 
-  shape_map = pango_x_get_shaper_map (lang);
+  shape_map = pango_x_get_shaper_map (language);
   return (PangoEngineShape *)pango_map_get_engine (shape_map, ch);
 }
 
@@ -1675,7 +1605,7 @@ pango_x_get_item_properties (PangoItem      *item,
 			     PangoAttrColor *bg_color,
 			     gboolean       *bg_set)
 {
-  GSList *tmp_list = item->extra_attrs;
+  GSList *tmp_list = item->analysis.extra_attrs;
 
   if (fg_set)
     *fg_set = FALSE;
